import 'dart:async';
import 'dart:developer';

import 'package:flutter/services.dart';
import 'package:hyperkyc_flutter/hyperkyc_config.dart';
import 'package:hyperkyc_flutter/hyperkyc_flutter_constants.dart';
import 'package:hyperkyc_flutter/hyperkyc_result.dart';

/// HyperKyc Flutter SDK can be used to create workflows to capture images of ID cards, photos of the face of the user, and perform other operations like face matches, etc all within itself to ease up integration friction on the client end.
class HyperKyc {
  /// Platform Method channel to perform HyperKyc related tasks
  static const MethodChannel _channel =
      MethodChannel(HyperKycConstants.hyperKycChannel);

  /// Generates a unique ID using the HyperKyc platform.
  ///
  /// This function invokes the HyperKyc channel's method [createUniqueId]
  /// to obtain a unique identifier. It returns a [Future] that completes
  /// with a [String] representing the generated unique ID.
  ///
  /// In case of an error during the method invocation, an error message
  /// is logged using the [log] function from the 'dart:developer' library.
  ///
  /// Example usage:
  /// ```dart
  /// String uniqueId = await createUniqueId();
  /// print('Generated Unique ID: $uniqueId');
  /// ```
  ///
  /// Returns a [Future] that completes with the generated unique ID.
  static Future<String> createUniqueId() async {
    String uniqueId = '';
    try {
      uniqueId = await _channel.invokeMethod(HyperKycConstants.createUniqueId);
    } catch (e) {
      log(e.toString());
    }
    return uniqueId;
  }

  /// Launches the HyperKyc SDK using the provided [HyperKycConfig] instance.
  ///
  /// This function initiates the HyperKyc SDK with the specified configuration
  /// provided through the [hyperKycConfig] parameter. It returns a [Future] that
  /// completes with a [HyperKycResult] representing the result of the SDK launch.
  ///
  /// The [HyperKycResult] contains information about the outcome of the SDK operation.
  /// Please refer to the Hyperverge team's documentation for details on how to process
  /// the [hyperKycResult] and handle different scenarios.
  ///
  /// Example usage:
  /// ```dart
  /// HyperKycConfig config = HyperKycConfig(/* provide configuration parameters */);
  /// HyperKycResult result = await HyperKycSDK.launch(hyperKycConfig: config);
  /// print('HyperKyc SDK Launch Result: $result');
  /// ```
  ///
  /// Returns a [Future] that completes with the [HyperKycResult] of the SDK launch.
  static Future<HyperKycResult> launch(
      {required HyperKycConfig hyperKycConfig}) async {
    Map hyperKycResultMap = {};
    HyperKycResult hyperKycResult = HyperKycResult();
    try {
      hyperKycResultMap = await _channel.invokeMethod(
        HyperKycConstants.launch,
        {
          HyperKycConstants.hyperKycConfig: hyperKycConfig.toMap(),
        },
      );
      hyperKycResult = HyperKycResult.fromMap(hyperKycResultMap);
    } catch (e) {
      log(e.toString());
    }
    return hyperKycResult;
  }
}
